#!/usr/bin/perl
#
# ==========================================================================
# Description:   Simple script to generate ASCII control codes from
#                ^B (bold), ^C (color), ^P (plain), ^R (reverse) and
#                ^U (underline) formatting symbols. You can escape
#                a caret (^) with doubling it; for example, ^^ is
#                converted to ^, and ^^C to ^C.
# ==========================================================================
# Author:        AngryWolf <angrywolf@flashmail.com>
# ==========================================================================
# Note:          If you intend to use this script under Windows, then you
#                need ActivePerl to run it, which is downloadable at
#                http://www.perl.com/pub/a/language/info/software.html#win32
#
#                You can use and modify this file for free, without
#                asking for permission.
# ==========================================================================
# Changes:
#
# $Log: convert.pl,v $
# Revision 1.2  2004/03/07 12:58:15  angrywolf
# - Now you can escape formatting symbols (thanks to codemastr for
#   reminding me about the importance of that).
#
# Revision 1.1  2004/03/07 12:57:57  angrywolf
# Initial revision
#
# ==========================================================================
# $Id: convert.pl,v 1.2 2004/03/07 12:58:15 angrywolf Exp $
# ==========================================================================

%symbols =
(
	'B'	=> chr(2),	# bold
	'C'	=> chr(3),	# color
	'P'	=> chr(15),	# plain
	'R'	=> chr(22),	# reverse
	'U'	=> chr(31)	# underline
);

if (!$ARGV[1])
{
	print "Usage: convert.pl <file-in> <file-out>\n";
	exit 1;
}

$in = shift @ARGV;
$out = shift @ARGV;

open(IN, $in) || die "Unable to open file $in for reading: $!";
open(OUT, ">$out") || die "Unable to open file $out for writing: $!";

while ($line = <IN>)
{
	my $tmp = "";
	@line = split(//s, $line);

	for ($i = 0; $i <= $#line; $i++)
	{
		if ($line[$i] ne '^' || $i == $#line) {
			$tmp .= $line[$i];
		} else {
			if ($line[$i+1] eq '^') {
				$tmp .= '^';
			} elsif (defined $symbols{$line[$i+1]}) {
				$tmp .= $symbols{$line[$i+1]};
			} else {
				$tmp .= '^' . $line[$i+1];
			}
			$i++;
		}
	}
	print OUT $tmp;
}

close(IN);
close(OUT);
exit 0;
